//+------------------------------------------------------------------+
//|                                           EGEN Trend Advisor.mq4 |
//|                             Copyright  2009, EGEN Software LTD. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+

#property copyright "Copyright  2009, EGEN Software LTD."
#property link      "http://www.metaquotes.net"

#define  SIZE     625
#define  OP_ALL   6

extern   bool     CloseAll          =  false;
extern   bool     Shutdown          =  false;
extern   double   TakeProfit        =  0;
extern   double   StopLoss          =  0;
extern   double   Lots              =  0;
extern   int      MaxOrders         =  100;
extern   int      Slippage          =  3;

int      TICKET[OP_ALL];
int      MAGIC;
double   ARRAY[SIZE][SIZE];
datetime TIME;

bool EQ(int value1, int value2)
{
   return(value1==value2);
}

datetime TIME()
{
   return(iTime(NULL,0,0));
}

int TREND(int mode)
{
   return(iCustom(NULL,0,"iTrend",mode,0));
}

int FAST()
{
   return(TREND(0));
}

int SLOW()
{
   return(TREND(1));
}

int MAGIC()
{
   int F=(FAST()+12)*25;
   int S=(SLOW()+12);
   return(F+S);   
}

double INFO(int type)
{
   return(MarketInfo(Symbol(),type));
}

double STOPLEVEL()
{
   return(INFO(MODE_STOPLEVEL)*Point);
}

double MINLOT()
{
   return(INFO(MODE_MINLOT));
}

double LEVEL(int factor)
{
   return(NormalizeDouble(STOPLEVEL()*factor,Digits));
}

double ASK(int factor=1)
{
   return(Ask+LEVEL(factor));
}

double BID(int factor=1)
{
   return(Bid-LEVEL(factor));
}

bool TOTAL(int index)
{
   return(index<OrdersTotal());
}

bool SYMBOL()
{
   return(EQ(OrderSymbol(),Symbol()));
}

bool SelectOrder(int index) 
{
   if(OrderSelect(index,SELECT_BY_POS)) 
      return(SYMBOL());
   return(false);         
}

double LOTS()
{   
   if(Lots>0) return(Lots); else return(MINLOT());
}

bool CheckMagic()
{  
   for(int i=0; TOTAL(i); i++) 
      if(SelectOrder(i))
         if(EQ(OrderMagicNumber(),MAGIC)) return(true);
      else continue;         
   return(false);      
}

void Buy()
{ 
   OrderSend(Symbol(),OP_BUYLIMIT,LOTS(),BID(),Slippage,0,0,NULL,MAGIC,0,Blue); 
}

void Sell()
{
   OrderSend(Symbol(),OP_SELLLIMIT,LOTS(),ASK(),Slippage,0,0,NULL,MAGIC,0,Red);   
}

int ORDER()
{   
   if(EQ(OrderType(),OP_SELL))       
      return(-1); 
   if(EQ(OrderType(),OP_BUY)) 
      return(1); 
   return(0);      
}  

double PRICE()
{
   if(ORDER()>0) return(Bid);
      else 
   if(ORDER()<0) return(Ask);     
}

void CloseTicket()
{
   OrderClose(OrderTicket(),OrderLots(),PRICE(),Slippage,White);           
}

void CloseOrder(int index)
{  
   if(SelectOrder(index)) CloseTicket();
}

void CloseAll()
{
   for(int i=0; TOTAL(i); i++) CloseOrder(i);
}

bool TakeProfit()
{
   if(TakeProfit>0)
      return(AccountEquity()>TakeProfit); 
   return(false);      
}

bool StopLoss()
{
   if(StopLoss>0)
      return(AccountEquity()<StopLoss);
   return(false);      
}

double DELTA(int magic)
{
   double delta=ARRAY[MAGIC][magic];
   if(MathAbs(delta)>0) return(delta);
   delta=ARRAY[magic][magic];
   return(delta);   
}
 
void SendOrder()
{
   if(MAGIC<0) MAGIC=MAGIC();
   if(TIME()>TIME) TIME=TIME(); else return;
   int magic=MAGIC();
   if(EQ(MAGIC,magic)) return;
   double delta=DELTA(magic);      
   MAGIC=magic;
   if(CheckMagic()) return;
   if(OrdersTotal()<MaxOrders)  
      if(delta<0) Sell(); else if(delta>0) Buy();
   else return;      
}

int BUY()
{
   return(TICKET[OP_BUY]);
}

int SELL()
{
   return(TICKET[OP_SELL]);
}

bool CheckTicket()
{
   int op=OrderType();
   if(TICKET[op]<0)
      TICKET[op]=OrderTicket();
   if(BUY()<0)
      return(false);
   if(SELL()<0)
      return(false);
   return(true);            
}

void ClearTicket()
{
   for(int op=OP_BUY; op<OP_ALL; op++) TICKET[op]=EMPTY;
}

bool CheckHedge(int shift)
{
   ClearTicket();
   for(int i=shift; TOTAL(i); i++)
      if(SelectOrder(i))
         if(CheckTicket()) return(true);
      else continue;
   return(false);      
}

void CloseHedge()
{
   OrderCloseBy(BUY(),SELL(),White);           
}

void RemoveOrder(int index)
{
   if(CheckHedge(index)) CloseHedge();
}

void RemoveAll()
{
   for(int i=0; TOTAL(i); i++) RemoveOrder(i);      
}

void CheckTrade()
{
   if(CloseAll) 
      CloseAll(); else
   if(Shutdown)
      CloseAll(); else      
   if(TakeProfit()) 
      CloseAll=true; else
   if(StopLoss()) 
      CloseAll=true; else 
   SendOrder();  
   RemoveAll();   
   Comment(DoubleToStr(AccountProfit(),2));
}
  
void init()
{
   MAGIC=EMPTY; 
   ArrayInitialize(ARRAY,0);
   int file=FileOpen(Symbol()+".BIN",FILE_BIN|FILE_READ);
   FileReadArray(file,ARRAY,0,ArraySize(ARRAY));
   FileClose(file);  
}
  
void start()
{
   if(IsTradeAllowed()) CheckTrade();
}


